@@ -146,23 +146,23 @@ static void handle_dahdi_exception(struct e1inp_ts *ts)
 	case DAHDI_EVENT_ALARM:
 		/* we should notify the code that the line is gone */
 		osmo_signal_dispatch(SS_L_INPUT, S_L_INP_LINE_ALARM, &isd);
-		rate_ctr_inc(&line->rate_ctr->ctr[E1I_CTR_ALARM]);
+		rate_ctr_inc(rate_ctr_group_get_ctr(line->rate_ctr, E1I_CTR_ALARM));
 		break;
 	case DAHDI_EVENT_NOALARM:
 		/* alarm has gone, we should re-start the SABM requests */
 		osmo_signal_dispatch(SS_L_INPUT, S_L_INP_LINE_NOALARM, &isd);
 		break;
 	case DAHDI_EVENT_ABORT:
-		rate_ctr_inc(&line->rate_ctr->ctr[E1I_CTR_HDLC_ABORT]);
+		rate_ctr_inc(rate_ctr_group_get_ctr(line->rate_ctr, E1I_CTR_HDLC_ABORT));
 		break;
 	case DAHDI_EVENT_OVERRUN:
-		rate_ctr_inc(&line->rate_ctr->ctr[E1I_CTR_HDLC_OVERR]);
+		rate_ctr_inc(rate_ctr_group_get_ctr(line->rate_ctr, E1I_CTR_HDLC_OVERR));
 		break;
 	case DAHDI_EVENT_BADFCS:
-		rate_ctr_inc(&line->rate_ctr->ctr[E1I_CTR_HDLC_BADFCS]);
+		rate_ctr_inc(rate_ctr_group_get_ctr(line->rate_ctr, E1I_CTR_HDLC_BADFCS));
 		break;
 	case DAHDI_EVENT_REMOVED:
-		rate_ctr_inc(&line->rate_ctr->ctr[E1I_CTR_REMOVED]);
+		rate_ctr_inc(rate_ctr_group_get_ctr(line->rate_ctr, E1I_CTR_REMOVED));
 		break;
 	}
 }
