@@ -1,6 +1,6 @@
 /* OpenBSC Abis input driver for ip.access */
 
-/* (C) 2009 by Harald Welte <laforge@gnumonks.org>
+/* (C) 2009-2021 by Harald Welte <laforge@gnumonks.org>
  * (C) 2010 by Holger Hans Peter Freyther
  * (C) 2010 by On-Waves
  *
@@ -51,6 +51,9 @@
 #include <osmocom/core/backtrace.h>
 #include <osmocom/gsm/ipa.h>
 
+/* global parameters of IPA input driver */
+struct ipa_pars g_e1inp_ipaccess_pars;
+
 static void *tall_ipa_ctx;
 
 #define TS1_ALLOC_SIZE	900
@@ -1002,6 +1005,7 @@ err:
 
 struct ipaccess_line {
 	bool line_already_initialized;
+	struct ipa_client_conn *ipa_cli[NUM_E1_TS]; /* 0=OML, 1+N=TRX_N */
 };
 
 static int ipaccess_line_update(struct e1inp_line *line)
@@ -1018,14 +1022,11 @@ static int ipaccess_line_update(struct e1inp_line *line)
 	}
 	il = line->driver_data;
 
-	/* We only initialize this line once. */
-	if (il->line_already_initialized)
-		return 0;
-
-	il->line_already_initialized = true;
-
 	switch(line->ops->cfg.ipa.role) {
 	case E1INP_LINE_R_BSC: {
+		/* We only initialize this line once. */
+		if (il->line_already_initialized)
+			return 0;
 		struct ipa_server_link *oml_link, *rsl_link;
 		const char *ipa = e1inp_ipa_get_bind_addr();
 
@@ -1041,6 +1042,8 @@ static int ipaccess_line_update(struct e1inp_line *line)
 				"BSC link: %s\n", strerror(errno));
 			return -ENOMEM;
 		}
+		oml_link->dscp = g_e1inp_ipaccess_pars.oml.dscp;
+		oml_link->priority = g_e1inp_ipaccess_pars.oml.priority;
 		if (ipa_server_link_open(oml_link) < 0) {
 			LOGP(DLINP, LOGL_ERROR, "cannot open OML BSC link: %s\n",
 				strerror(errno));
@@ -1055,6 +1058,8 @@ static int ipaccess_line_update(struct e1inp_line *line)
 				"BSC link: %s\n", strerror(errno));
 			return -ENOMEM;
 		}
+		rsl_link->dscp = g_e1inp_ipaccess_pars.rsl.dscp;
+		rsl_link->priority = g_e1inp_ipaccess_pars.rsl.priority;
 		if (ipa_server_link_open(rsl_link) < 0) {
 			LOGP(DLINP, LOGL_ERROR, "cannot open RSL BSC link: %s\n",
 				strerror(errno));
@@ -1072,6 +1077,13 @@ static int ipaccess_line_update(struct e1inp_line *line)
 		     "OML connecting to %s:%u\n", line->ops->cfg.ipa.addr,
 		     IPA_TCP_PORT_OML);
 
+		/* Drop previous line */
+		if (il->ipa_cli[0]) {
+			ipa_client_conn_close(il->ipa_cli[0]);
+			ipa_client_conn_destroy(il->ipa_cli[0]);
+			il->ipa_cli[0] = NULL;
+		}
+
 		link = ipa_client_conn_create2(tall_ipa_ctx,
 					      e1inp_line_ipa_oml_ts(line),
 					      E1INP_SIGN_OML,
@@ -1087,6 +1099,8 @@ static int ipaccess_line_update(struct e1inp_line *line)
 				"BTS link: %s\n", strerror(errno));
 			return -ENOMEM;
 		}
+		link->dscp = g_e1inp_ipaccess_pars.oml.dscp;
+		link->priority = g_e1inp_ipaccess_pars.oml.priority;
 		if (ipa_client_conn_open(link) < 0) {
 			LOGP(DLINP, LOGL_ERROR, "cannot open OML BTS link: %s\n",
 				strerror(errno));
@@ -1097,12 +1111,15 @@ static int ipaccess_line_update(struct e1inp_line *line)
 
 		e1i_ts = e1inp_line_ipa_oml_ts(line);
 		ipaccess_bts_keepalive_fsm_alloc(e1i_ts, link, "oml_bts_to_bsc");
+		il->ipa_cli[0] = link;
 		ret = 0;
 		break;
 	}
 	default:
 		break;
 	}
+
+	il->line_already_initialized = true;
 	return ret;
 }
 
@@ -1120,6 +1137,7 @@ int e1inp_ipa_bts_rsl_connect_n(struct e1inp_line *line,
 {
 	struct ipa_client_conn *rsl_link;
 	struct e1inp_ts *e1i_ts = e1inp_line_ipa_rsl_ts(line, trx_nr);
+	struct ipaccess_line *il;
 
 	if (E1INP_SIGN_RSL+trx_nr-1 >= NUM_E1_TS) {
 		LOGP(DLINP, LOGL_ERROR, "cannot create RSL BTS link: "
@@ -1127,6 +1145,17 @@ int e1inp_ipa_bts_rsl_connect_n(struct e1inp_line *line,
 		return -EINVAL;
 	}
 
+	if (!line->driver_data)
+		line->driver_data = talloc_zero(line, struct ipaccess_line);
+	il = line->driver_data;
+
+	/* Drop previous line */
+	if (il->ipa_cli[1 + trx_nr]) {
+		ipa_client_conn_close(il->ipa_cli[1 + trx_nr]);
+		ipa_client_conn_destroy(il->ipa_cli[1 + trx_nr]);
+		il->ipa_cli[1 + trx_nr] = NULL;
+	}
+
 	rsl_link = ipa_client_conn_create2(tall_ipa_ctx,
 					  e1inp_line_ipa_rsl_ts(line, trx_nr),
 					  E1INP_SIGN_RSL+trx_nr,
@@ -1141,6 +1170,8 @@ int e1inp_ipa_bts_rsl_connect_n(struct e1inp_line *line,
 			"BTS link: %s\n", strerror(errno));
 		return -ENOMEM;
 	}
+	rsl_link->dscp = g_e1inp_ipaccess_pars.rsl.dscp;
+	rsl_link->priority = g_e1inp_ipaccess_pars.rsl.priority;
 	if (ipa_client_conn_open(rsl_link) < 0) {
 		LOGP(DLINP, LOGL_ERROR, "cannot open RSL BTS link: %s\n",
 			strerror(errno));
@@ -1148,8 +1179,8 @@ int e1inp_ipa_bts_rsl_connect_n(struct e1inp_line *line,
 		ipa_client_conn_destroy(rsl_link);
 		return -EIO;
 	}
-
 	ipaccess_bts_keepalive_fsm_alloc(e1i_ts, rsl_link, "rsl_bts_to_bsc");
+	il->ipa_cli[1 + trx_nr] = rsl_link;
 	return 0;
 }
 
